@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@xwax.org>
+ * Copyright (C) 2013 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -20,12 +20,14 @@
 #ifndef DEVICE_H
 #define DEVICE_H
 
+#include <stdbool.h>
 #include <sys/poll.h>
 #include <sys/types.h>
 
 #define DEVICE_CHANNELS 2
 
 struct device {
+    bool fault;
     void *local;
     struct device_ops *ops;
 
@@ -44,6 +46,9 @@ struct device_ops {
     void (*clear)(struct device *dv);
 };
 
+void device_init(struct device *dv, struct device_ops *ops);
+void device_clear(struct device *dv);
+
 void device_connect_timecoder(struct device *dv, struct timecoder *tc);
 void device_connect_player(struct device *dv, struct player *pl);
 
@@ -52,10 +57,8 @@ unsigned int device_sample_rate(struct device *dv);
 void device_start(struct device *dv);
 void device_stop(struct device *dv);
 
-void device_clear(struct device *dv);
-
 ssize_t device_pollfds(struct device *dv, struct pollfd *pe, size_t z);
-int device_handle(struct device *dv);
+void device_handle(struct device *dv);
 
 void device_submit(struct device *dv, signed short *pcm, size_t npcm);
 void device_collect(struct device *dv, signed short *pcm, size_t npcm);
