@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2012 Mark Hills <mark@pogo.org.uk>
+ * Copyright (C) 2013 Mark Hills <mark@xwax.org>
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License
@@ -28,6 +28,10 @@ void rig_clear();
 int rig_main();
 
 int rig_quit();
+
+void rig_lock();
+void rig_unlock();
+
 void rig_post_track(struct track *t);
 
 #endif
